#include <sys/types.h>
#include <sys/stat.h>
#include <termios.h>
#include <stdio.h>
#include <fcntl.h>

#include "qcam.h"
#include "qcamip.h"
#include <jpeglib.h>
#include "writejpeg.h"

#include <vga.h>
#include <vgamouse.h>
#include <vgakeyboard.h>

#define INPUTDEVICE "/dev/stdin"

#define DIM_X 320
#define DIM_Y 200

/*
with libqcam.a and libqcamip.a files in
current working directory
gcc -O2 -o view vfj.c writejpeg.c -L. -lqcam -lqcamip -lvga -ljpeg

*/

void main (void) {

  scanbuf *scanBuf;
  struct qcam *q;
  extern int image_width, image_height, image_quality;
  int j, shift, gridsize;

  char *video_buff;
  int i;
  int k = 0; /* k is frame counter */
  char filename[ 80 ];

  FILE *fpSnapShot;

  struct termios oldtio, mytio;
  char STDINBuf[2];
  int fdSTDIN, STDINResult;

  fdSTDIN = open(INPUTDEVICE, O_RDONLY | O_NOCTTY | O_NONBLOCK);
  if (fdSTDIN<0) {perror(INPUTDEVICE); exit(-1);}

  tcgetattr(fdSTDIN,&oldtio);
  bzero(&mytio,sizeof(mytio));
  mytio.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
  mytio.c_iflag = IGNPAR;
  mytio.c_oflag = 0;
  mytio.c_lflag = 0;
  mytio.c_cc[VTIME] = .02;
  mytio.c_cc[VMIN] = 0;

  tcflush(fdSTDIN,TCIOFLUSH);
  tcsetattr(fdSTDIN,TCSANOW,&mytio);
  tcflow(fdSTDIN,TCOON);

  /* set decent image quality for JPEG compression */
  image_quality = 50;

  /* SVGAlib mouse init */
  vga_setmousesupport( 1 );

  /* configure SVGAlib */
  vga_init();
  vga_setmode( G320x200x256 );
  video_buff = vga_getgraphmem();

  /* set up greyscale palette */
  vga_waitretrace();
  for( i = 0; i < 64; i++ ) {
    vga_setpalette( i, i, i, i );
  }
  for( i = 64; i < 128; i++ ) {
    vga_setpalette( i, i, 0, 0 );
  }
  for( i = 128; i < 192; i++ ) {
    vga_setpalette( i, 0, i, 0 );
  }
  for( i = 192; i < 255; i++ ) {
    vga_setpalette( i, 0, 0, i );
  }


  q = qc_init ();		/* set built in defaults, init qcam pointer */
  qc_initfile (q, NULL);	/* read defaults from file */

  /* Dimensions of our image- useful later */
  image_width = qc_getwidth(q) / qc_gettransfer_scale(q);
  image_height = qc_getheight(q) / qc_gettransfer_scale(q);
  gridsize = image_width * image_height;
  shift = BITS_IN_JSAMPLE - qc_getbitdepth(q);

  /* Prevent segmentation fault if qcam is on an unsupported resolution */
  if ((image_width < 320) || (image_height < 200)) {
    fprintf (stderr, "QCAM resolution must be set to at least 320X200 - change /usr/local/etc/qcam.conf\n");
    fprintf (stderr, "QCAM resolution is currently set to %i X %i\n",image_width,image_height);
    exit (-1);
  }
	  
  if (qc_open (q)) {	/* locking QCAM device */
    fprintf (stderr, "An error occured opening QCAM.\n");
    exit (-1);
  }



  while(1) {

    STDINResult = read(fdSTDIN,STDINBuf,1);

    qc_set (q);		/* reset, set SendFrameMode, init QCAM */
    scanBuf = qc_scan (q);	/* get a frame from QCam */

      if( ((STDINResult > 0) && ((STDINBuf[0] == 'o') || (STDINBuf[0] == 'O')) ) ) {
        sprintf( filename, "./v%03d.jpeg", k,STDINBuf );
        fpSnapShot = fopen( filename, "w" ); 

	/*  Correction to prevent dark pictures (I think) */
        for (j=0;j<gridsize;j++) {
          scanBuf[j] = scanBuf[j] << shift;
        }

        write_JPEG_file((JSAMPLE *) scanBuf, fpSnapShot);	
        fclose( fpSnapShot );
        k++ ; /* increment frame counter */
        }
      if( ((STDINResult > 0) && ((STDINBuf[0] == 's') || (STDINBuf[0] == 'S')) ) ) {
	system("./sendPics");
      }
	

/* By using the modified scanBuf (corrected to prevent dark pictures), we get a bit of a flash on the screen whenever a picture is taken.  This could be prevented by using a separate scanbuf for the screen, but I kind of like the feedback of a "flash" */

/* WARNING: This line causes a segmentation fault if the qcam is not set to 320X200 or greater resolution.  We should copy line by line and add black to the end of each line if it doesn't fill the whole screen.  Maybe later.  Once I have time to see if there's a performance hit.. (probably not since the parallel port is so slow..) */

    memcpy( video_buff, scanBuf, DIM_X * DIM_Y );

    free (scanBuf);		/* release memory */

  }


  qc_close (q);		/* clear locks */
}

